@@ -129,6 +129,12 @@ private Q_SLOTS:
 
     void testSaveAndRestoreSettings();
 
+    void testSaveLoadPlayList();
+
+    void testEnqueueFiles();
+
+    void testEnqueueSampleFiles();
+
 private:
 
     QList<MusicAudioTrack> mNewTracks;
