@@ -76,6 +76,8 @@ public:
     qulonglong trackIdFromTitleAlbumTrackDiscNumber(const QString &title, const QString &artist, const QString &album,
                                                     int trackNumber, int discNumber);
 
+    qulonglong trackIdFromFileName(const QUrl &fileName);
+
     void applicationAboutToQuit();
 
 Q_SIGNALS:
@@ -110,7 +112,7 @@ public Q_SLOTS:
 
     void removeTracksList(const QList<QUrl> &removedTracks);
 
-    void modifyTracksList(const QList<MusicAudioTrack> &modifiedTracks, const QHash<QString, QUrl> &covers);
+    void modifyTracksList(const QList<MusicAudioTrack> &modifiedTracks, const QHash<QString, QUrl> &covers, const QString &musicSource);
 
     void removeAllTracksFromSource(const QString &sourceName);
 
@@ -195,6 +197,10 @@ private:
 
     void internalRemoveTracksWithoutMapping();
 
+    QUrl internalAlbumArtUriFromAlbumId(qulonglong albumId);
+
+    bool isValidArtist(qulonglong albumId);
+
     std::unique_ptr<DatabaseInterfacePrivate> d;
 
 };
