@@ -25,6 +25,9 @@
 #include <QUrl>
 #include <QPersistentModelIndex>
 #include <QList>
+#include <QMediaPlaylist>
+#include <QFileInfo>
+#include <QDebug>
 
 #include <algorithm>
 
@@ -48,10 +51,14 @@ public:
 
     QVariantMap mPersistentState;
 
+    QMediaPlaylist mLoadPlaylist;
+
 };
 
 MediaPlayList::MediaPlayList(QObject *parent) : QAbstractListModel(parent), d(new MediaPlayListPrivate)
 {
+    connect(&d->mLoadPlaylist, &QMediaPlaylist::loaded, this, &MediaPlayList::loadPlayListLoaded);
+    connect(&d->mLoadPlaylist, &QMediaPlaylist::loadFailed, this, &MediaPlayList::loadPlayListLoadFailed);
 }
 
 MediaPlayList::~MediaPlayList()
@@ -66,6 +73,41 @@ int MediaPlayList::rowCount(const QModelIndex &parent) const
     return d->mData.size();
 }
 
+QHash<int, QByteArray> MediaPlayList::roleNames() const
+{
+    auto roles = QAbstractItemModel::roleNames();
+
+    roles[static_cast<int>(ColumnsRoles::IsValidRole)] = "isValid";
+    roles[static_cast<int>(ColumnsRoles::TitleRole)] = "title";
+    roles[static_cast<int>(ColumnsRoles::DurationRole)] = "duration";
+    roles[static_cast<int>(ColumnsRoles::ArtistRole)] = "artist";
+    roles[static_cast<int>(ColumnsRoles::AlbumArtistRole)] = "albumArtist";
+    roles[static_cast<int>(ColumnsRoles::AlbumRole)] = "album";
+    roles[static_cast<int>(ColumnsRoles::TrackNumberRole)] = "trackNumber";
+    roles[static_cast<int>(ColumnsRoles::DiscNumberRole)] = "discNumber";
+    roles[static_cast<int>(ColumnsRoles::RatingRole)] = "rating";
+    roles[static_cast<int>(ColumnsRoles::GenreRole)] = "genre";
+    roles[static_cast<int>(ColumnsRoles::LyricistRole)] = "lyricist";
+    roles[static_cast<int>(ColumnsRoles::ComposerRole)] = "composer";
+    roles[static_cast<int>(ColumnsRoles::CommentRole)] = "comment";
+    roles[static_cast<int>(ColumnsRoles::YearRole)] = "year";
+    roles[static_cast<int>(ColumnsRoles::ChannelsRole)] = "channels";
+    roles[static_cast<int>(ColumnsRoles::BitRateRole)] = "bitRate";
+    roles[static_cast<int>(ColumnsRoles::SampleRateRole)] = "sampleRate";
+    roles[static_cast<int>(ColumnsRoles::ImageRole)] = "image";
+    roles[static_cast<int>(ColumnsRoles::CountRole)] = "count";
+    roles[static_cast<int>(ColumnsRoles::IsPlayingRole)] = "isPlaying";
+    roles[static_cast<int>(ColumnsRoles::HasAlbumHeader)] = "hasAlbumHeader";
+    roles[static_cast<int>(ColumnsRoles::IsSingleDiscAlbumHeader)] = "isSingleDiscAlbum";
+    roles[static_cast<int>(ColumnsRoles::SecondaryTextRole)] = "secondaryText";
+    roles[static_cast<int>(ColumnsRoles::ImageUrlRole)] = "imageUrl";
+    roles[static_cast<int>(ColumnsRoles::ShadowForImageRole)] = "shadowForImage";
+    roles[static_cast<int>(ColumnsRoles::ResourceRole)] = "trackResource";
+    roles[static_cast<int>(ColumnsRoles::TrackDataRole)] = "trackData";
+
+    return roles;
+}
+
 QVariant MediaPlayList::data(const QModelIndex &index, int role) const
 {
     auto result = QVariant();
@@ -78,20 +120,23 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
         return result;
     }
 
-    if (role < ColumnsRoles::IsValidRole || role > ColumnsRoles::IsSingleDiscAlbumHeader) {
-        return result;
-    }
-
-    ColumnsRoles convertedRole = static_cast<ColumnsRoles>(role);
-
     if (d->mData[index.row()].mIsValid) {
-        switch(convertedRole)
+        switch(role)
         {
         case ColumnsRoles::IsValidRole:
             result = d->mData[index.row()].mIsValid;
             break;
         case ColumnsRoles::TitleRole:
-            result = d->mTrackData[index.row()].title();
+            if (!d->mTrackData[index.row()].title().isEmpty()) {
+                result = d->mTrackData[index.row()].title();
+            } else {
+                if (d->mData[index.row()].mTrackUrl.isLocalFile()) {
+                    auto localFile = QFileInfo(d->mData[index.row()].mTrackUrl.toLocalFile());
+                    result = localFile.fileName();
+                } else {
+                    result = d->mData[index.row()].mTrackUrl.toString();
+                }
+            }
             break;
         case ColumnsRoles::DurationRole:
         {
@@ -119,15 +164,17 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
             result = d->mTrackData[index.row()].trackNumber();
             break;
         case ColumnsRoles::DiscNumberRole:
-            if (d->mTrackData[index.row()].discNumber() > 0) {
-                result = d->mTrackData[index.row()].discNumber();
-            }
+            result = d->mTrackData[index.row()].discNumber();
             break;
         case ColumnsRoles::IsSingleDiscAlbumHeader:
             result = d->mTrackData[index.row()].isSingleDiscAlbum();
             break;
         case ColumnsRoles::ResourceRole:
-            result = d->mTrackData[index.row()].resourceURI();
+            if (d->mTrackData[index.row()].resourceURI().isValid()) {
+                result = d->mTrackData[index.row()].resourceURI();
+            } else {
+                result = d->mData[index.row()].mTrackUrl;
+            }
             break;
         case ColumnsRoles::ImageRole:
         {
@@ -143,22 +190,91 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
         case ColumnsRoles::RatingRole:
             result = d->mTrackData[index.row()].rating();
             break;
-        case ColumnsRoles::CountRole:
+        case ColumnsRoles::GenreRole:
+            result = d->mTrackData[index.row()].genre();
             break;
-        case ColumnsRoles::CreatorRole:
+        case ColumnsRoles::LyricistRole:
+            result = d->mTrackData[index.row()].lyricist();
+            break;
+        case ColumnsRoles::ComposerRole:
+            result = d->mTrackData[index.row()].composer();
+            break;
+        case ColumnsRoles::CommentRole:
+            result = d->mTrackData[index.row()].comment();
+            break;
+        case ColumnsRoles::YearRole:
+            result = d->mTrackData[index.row()].year();
+            break;
+        case ColumnsRoles::ChannelsRole:
+            result = d->mTrackData[index.row()].channels();
+            break;
+        case ColumnsRoles::BitRateRole:
+            result = d->mTrackData[index.row()].bitRate();
+            break;
+        case ColumnsRoles::SampleRateRole:
+            result = d->mTrackData[index.row()].sampleRate();
+            break;
+        case ColumnsRoles::CountRole:
             break;
         case ColumnsRoles::IsPlayingRole:
             result = d->mData[index.row()].mIsPlaying;
             break;
+        case Qt::DisplayRole:
+        {
+            const auto &track = d->mTrackData[index.row()];
+            auto displayText = QString();
+            displayText = QStringLiteral("%1 - %2");
+
+            if (track.isSingleDiscAlbum()) {
+                displayText = displayText.arg(track.trackNumber());
+            } else {
+                auto numbersText = QString();
+                numbersText = QStringLiteral("%1 - %2");
+                numbersText = numbersText.arg(track.discNumber());
+                numbersText = numbersText.arg(track.trackNumber());
+                displayText = displayText.arg(numbersText);
+            }
+
+            result = displayText.arg(track.title());
+            break;
         }
+        case ColumnsRoles::SecondaryTextRole:
+            break;
+        case ColumnsRoles::ImageUrlRole:
+        {
+            const auto &albumArt = d->mTrackData[index.row()].albumCover();
+            if (albumArt.isValid()) {
+                result = albumArt;
+            } else {
+                result = QUrl(QStringLiteral("image://icon/media-optical-audio"));
+            }
+            break;
+        }
+        case ColumnsRoles::ShadowForImageRole:
+            result = d->mTrackData[index.row()].albumCover().isValid();
+            break;
+        case ColumnsRoles::TrackDataRole:
+            result = QVariant::fromValue(d->mTrackData[index.row()]);
+            break;
+        }
+
     } else {
-        switch(convertedRole)
+        switch(role)
         {
         case ColumnsRoles::IsValidRole:
             result = d->mData[index.row()].mIsValid;
             break;
         case ColumnsRoles::TitleRole:
-            result = d->mData[index.row()].mTitle;
+            if (!d->mData[index.row()].mTitle.isEmpty()) {
+                result = d->mData[index.row()].mTitle;
+            } else if (d->mData[index.row()].mTrackUrl.isValid()) {
+                if (d->mData[index.row()].mTrackUrl.isLocalFile()) {
+                    auto localFile = QFileInfo(d->mData[index.row()].mTrackUrl.toLocalFile());
+                    result = localFile.fileName();
+                } else {
+                    result = d->mData[index.row()].mTrackUrl.toString();
+                }
+            }
             break;
         case ColumnsRoles::IsPlayingRole:
             result = d->mData[index.row()].mIsPlaying;
@@ -192,11 +308,21 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
             break;
         case ColumnsRoles::CountRole:
             break;
-        case ColumnsRoles::CreatorRole:
-            break;
         case ColumnsRoles::ImageRole:
             result = QStringLiteral("");
             break;
+        case Qt::DisplayRole:
+            result = d->mTrackData[index.row()].title();
+            break;
+        case ColumnsRoles::SecondaryTextRole:
+            result = QString();
+            break;
+        case ColumnsRoles::ImageUrlRole:
+            result = QUrl(QStringLiteral("image://icon/error"));
+            break;
+        case ColumnsRoles::ShadowForImageRole:
+            result = false;
+            break;
         }
     }
 
@@ -243,28 +369,6 @@ bool MediaPlayList::setData(const QModelIndex &index, const QVariant &value, int
     return modelModified;
 }
 
-QHash<int, QByteArray> MediaPlayList::roleNames() const
-{
-    QHash<int, QByteArray> roles;
-
-    roles[static_cast<int>(ColumnsRoles::IsValidRole)] = "isValid";
-    roles[static_cast<int>(ColumnsRoles::TitleRole)] = "title";
-    roles[static_cast<int>(ColumnsRoles::DurationRole)] = "duration";
-    roles[static_cast<int>(ColumnsRoles::ArtistRole)] = "artist";
-    roles[static_cast<int>(ColumnsRoles::AlbumArtistRole)] = "albumArtist";
-    roles[static_cast<int>(ColumnsRoles::AlbumRole)] = "album";
-    roles[static_cast<int>(ColumnsRoles::TrackNumberRole)] = "trackNumber";
-    roles[static_cast<int>(ColumnsRoles::DiscNumberRole)] = "discNumber";
-    roles[static_cast<int>(ColumnsRoles::RatingRole)] = "rating";
-    roles[static_cast<int>(ColumnsRoles::ImageRole)] = "image";
-    roles[static_cast<int>(ColumnsRoles::CountRole)] = "count";
-    roles[static_cast<int>(ColumnsRoles::IsPlayingRole)] = "isPlaying";
-    roles[static_cast<int>(ColumnsRoles::HasAlbumHeader)] = "hasAlbumHeader";
-    roles[static_cast<int>(ColumnsRoles::IsSingleDiscAlbumHeader)] = "isSingleDiscAlbum";
-
-    return roles;
-}
-
 bool MediaPlayList::removeRows(int row, int count, const QModelIndex &parent)
 {
     beginRemoveRows(parent, row, row + count - 1);
@@ -358,7 +462,18 @@ void MediaPlayList::enqueue(const MediaPlayListEntry &newEntry, const MusicAudio
     Q_EMIT persistentStateChanged();
 
     if (!newEntry.mIsValid) {
-        Q_EMIT newTrackByNameInList(newEntry.mTitle, newEntry.mArtist, newEntry.mAlbum, newEntry.mTrackNumber, newEntry.mDiscNumber);
+        if (newEntry.mTrackUrl.isValid()) {
+            qDebug() << "MediaPlayList::enqueue" << "newTrackByFileNameInList" << newEntry.mTrackUrl;
+            if (newEntry.mTrackUrl.isLocalFile()) {
+                QFileInfo newTrackFile(newEntry.mTrackUrl.toLocalFile());
+                if (newTrackFile.exists()) {
+                    d->mData.last().mIsValid = true;
+                }
+                Q_EMIT newTrackByFileNameInList(newEntry.mTrackUrl);
+            }
+        } else {
+            Q_EMIT newTrackByNameInList(newEntry.mTitle, newEntry.mArtist, newEntry.mAlbum, newEntry.mTrackNumber, newEntry.mDiscNumber);
+        }
     } else {
         Q_EMIT newTrackByIdInList(newEntry.mId);
     }
@@ -478,6 +593,11 @@ void MediaPlayList::enqueue(const MusicAlbum &album)
     }
 }
 
+void MediaPlayList::enqueue(const MusicArtist &artist)
+{
+    enqueue(artist.name());
+}
+
 void MediaPlayList::enqueue(const QString &artistName)
 {
     beginInsertRows(QModelIndex(), d->mData.size(), d->mData.size());
@@ -495,25 +615,50 @@ void MediaPlayList::enqueue(const QString &artistName)
     Q_EMIT persistentStateChanged();
 }
 
+void MediaPlayList::enqueue(const QUrl &fileName)
+{
+    qDebug() << "MediaPlayList::enqueue" << fileName;
+    enqueue(MediaPlayListEntry(fileName));
+}
+
+void MediaPlayList::enqueue(const QStringList &files)
+{
+    qDebug() << "MediaPlayList::enqueue" << files;
+    for (const auto &oneFileName : files) {
+        enqueue(QUrl::fromLocalFile(oneFileName));
+    }
+}
+
 void MediaPlayList::clearAndEnqueue(const MusicAlbum &album)
 {
     clearPlayList();
     enqueue(album);
 }
 
+void MediaPlayList::clearAndEnqueue(const MusicArtist &artist)
+{
+    clearAndEnqueue(artist.name());
+}
+
 void MediaPlayList::clearAndEnqueue(const QString &artistName)
 {
     clearPlayList();
     enqueue(artistName);
 }
 
+void MediaPlayList::clearAndEnqueue(const QUrl &fileName)
+{
+    clearPlayList();
+    enqueue(fileName);
+}
+
 void MediaPlayList::clearPlayList()
 {
     if (d->mData.isEmpty()) {
         return;
     }
 
-    beginRemoveRows({}, 0, d->mData.count());
+    beginRemoveRows({}, 0, d->mData.count() - 1);
     d->mData.clear();
     d->mTrackData.clear();
     endRemoveRows();
@@ -521,6 +666,27 @@ void MediaPlayList::clearPlayList()
     Q_EMIT tracksCountChanged();
 }
 
+void MediaPlayList::loadPlaylist(const QUrl &fileName)
+{
+    d->mLoadPlaylist.clear();
+    d->mLoadPlaylist.load(fileName, "m3u");
+}
+
+bool MediaPlayList::savePlaylist(const QUrl &fileName)
+{
+    QMediaPlaylist savePlaylist;
+
+    for (int i = 0; i < d->mData.size(); ++i) {
+        const auto &oneTrack = d->mData.at(i);
+        const auto &oneTrackData = d->mTrackData.at(i);
+        if (oneTrack.mIsValid) {
+            savePlaylist.addMedia(oneTrackData.resourceURI());
+        }
+    }
+
+    return savePlaylist.save(fileName, "m3u");
+}
+
 QVariantMap MediaPlayList::persistentState() const
 {
     auto currentState = QVariantMap();
@@ -673,7 +839,11 @@ void MediaPlayList::trackChanged(const MusicAudioTrack &track)
         auto &oneEntry = d->mData[i];
 
         if (!oneEntry.mIsArtist && oneEntry.mIsValid) {
-            if (track.databaseId() != oneEntry.mId) {
+            if (oneEntry.mTrackUrl.isValid() && track.resourceURI() != oneEntry.mTrackUrl) {
+                continue;
+            }
+
+            if (!oneEntry.mTrackUrl.isValid() && (oneEntry.mId == 0 || track.databaseId() != oneEntry.mId)) {
                 continue;
             }
 
@@ -687,7 +857,7 @@ void MediaPlayList::trackChanged(const MusicAudioTrack &track)
                 }
             }
             continue;
-        } else if (!oneEntry.mIsArtist && !oneEntry.mIsValid) {
+        } else if (!oneEntry.mIsArtist && !oneEntry.mIsValid && !oneEntry.mTrackUrl.isValid()) {
             if (track.title() != oneEntry.mTitle) {
                 continue;
             }
@@ -714,6 +884,26 @@ void MediaPlayList::trackChanged(const MusicAudioTrack &track)
                 resetCurrentTrack();
             }
 
+            break;
+        } else if (!oneEntry.mIsArtist && !oneEntry.mIsValid && oneEntry.mTrackUrl.isValid()) {
+            qDebug() << "MediaPlayList::trackChanged" << oneEntry << track;
+            qDebug() << "MediaPlayList::trackChanged" << track.resourceURI() << oneEntry.mTrackUrl;
+            if (track.resourceURI() != oneEntry.mTrackUrl) {
+                continue;
+            }
+
+            d->mTrackData[i] = track;
+            oneEntry.mId = track.databaseId();
+            oneEntry.mIsValid = true;
+
+            Q_EMIT dataChanged(index(i, 0), index(i, 0), {});
+
+            restorePlayListPosition();
+
+            if (!d->mCurrentTrack.isValid()) {
+                resetCurrentTrack();
+            }
+
             break;
         }
     }
@@ -904,6 +1094,30 @@ bool MediaPlayList::rowHasHeader(int row) const
     return true;
 }
 
+void MediaPlayList::loadPlayListLoaded()
+{
+    clearPlayList();
+
+    for (int i = 0; i < d->mLoadPlaylist.mediaCount(); ++i) {
+        enqueue(d->mLoadPlaylist.media(i).canonicalUrl());
+    }
+
+    restorePlayListPosition();
+    restoreRandomPlay();
+    restoreRepeatPlay();
+
+    Q_EMIT persistentStateChanged();
+
+    d->mLoadPlaylist.clear();
+    Q_EMIT playListLoaded();
+}
+
+void MediaPlayList::loadPlayListLoadFailed()
+{
+    d->mLoadPlaylist.clear();
+    Q_EMIT playListLoadFailed();
+}
+
 void MediaPlayList::resetCurrentTrack()
 {
     for(int row = 0; row < rowCount(); ++row) {
@@ -961,5 +1175,10 @@ void MediaPlayList::restoreRepeatPlay()
     }
 }
 
+QDebug operator<<(QDebug stream, const MediaPlayListEntry &data)
+{
+    stream << data.mTitle << data.mAlbum << data.mArtist << data.mTrackUrl << data.mTrackNumber << data.mDiscNumber << data.mId << data.mIsValid;
+    return stream;
+}
 
 #include "moc_mediaplaylist.cpp"
