@@ -36,10 +36,10 @@
 #include "trackslistener.h"
 #include "notificationitem.h"
 #include "elisaapplication.h"
-
-#if defined KF5Config_FOUND && KF5Config_FOUND
 #include "elisa_settings.h"
-#endif
+#include "allalbumsmodel.h"
+#include "allartistsmodel.h"
+#include "alltracksmodel.h"
 
 #include <KI18n/KLocalizedString>
 
@@ -73,9 +73,7 @@ public:
     QScopedPointer<BalooListener> mBalooListener;
 #endif
 
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
     std::list<std::unique_ptr<FileListener>> mFileListener;
-#endif
 
     DatabaseInterface mDatabaseInterface;
 
@@ -83,12 +81,22 @@ public:
 
     int mImportedTracksCount = 0;
 
+    int mTotalImportedTracksCount = 0;
+
     int mActiveMusicListenersCount = 0;
 
     bool mIndexingRunning = false;
 
     ElisaApplication *mElisaApplication = nullptr;
 
+    AllAlbumsModel mAllAlbumsModel;
+
+    AllArtistsModel mAllArtistsModel;
+
+    AllTracksModel mAllTracksModel;
+
+    bool mIndexerBusy = false;
+
 };
 
 MusicListenersManager::MusicListenersManager(QObject *parent)
@@ -137,7 +145,6 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
     connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
             this, &MusicListenersManager::applicationAboutToQuit);
 
-#if defined KF5Config_FOUND && KF5Config_FOUND
     connect(Elisa::ElisaConfiguration::self(), &Elisa::ElisaConfiguration::configChanged,
             this, &MusicListenersManager::configChanged);
 
@@ -146,7 +153,8 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
 
     auto initialRootPath = Elisa::ElisaConfiguration::rootPath();
     if (initialRootPath.isEmpty()) {
-        for (const auto &musicPath : QStandardPaths::standardLocations(QStandardPaths::MusicLocation)) {
+        auto systemMusicPaths = QStandardPaths::standardLocations(QStandardPaths::MusicLocation);
+        for (const auto &musicPath : qAsConst(systemMusicPaths)) {
             initialRootPath.push_back(musicPath);
         }
 
@@ -155,7 +163,30 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
     }
 
     d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
-#endif
+
+    d->mAllAlbumsModel.setAllArtists(&d->mAllArtistsModel);
+    d->mAllArtistsModel.setAllAlbums(&d->mAllAlbumsModel);
+
+    connect(&d->mDatabaseInterface, &DatabaseInterface::albumAdded,
+            &d->mAllAlbumsModel, &AllAlbumsModel::albumAdded);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::albumModified,
+            &d->mAllAlbumsModel, &AllAlbumsModel::albumModified);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::albumRemoved,
+            &d->mAllAlbumsModel, &AllAlbumsModel::albumRemoved);
+
+    connect(&d->mDatabaseInterface, &DatabaseInterface::artistAdded,
+            &d->mAllArtistsModel, &AllArtistsModel::artistAdded);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::artistModified,
+            &d->mAllArtistsModel, &AllArtistsModel::artistModified);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::artistRemoved,
+            &d->mAllArtistsModel, &AllArtistsModel::artistRemoved);
+
+    connect(&d->mDatabaseInterface, &DatabaseInterface::tracksAdded,
+            &d->mAllTracksModel, &AllTracksModel::tracksAdded);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::trackModified,
+            &d->mAllTracksModel, &AllTracksModel::trackModified);
+    connect(&d->mDatabaseInterface, &DatabaseInterface::trackRemoved,
+            &d->mAllTracksModel, &AllTracksModel::trackRemoved);
 }
 
 MusicListenersManager::~MusicListenersManager()
@@ -181,6 +212,7 @@ void MusicListenersManager::subscribeForTracks(MediaPlayList *client)
     connect(helper, &TracksListener::albumAdded, client, &MediaPlayList::albumAdded);
     connect(client, &MediaPlayList::newTrackByIdInList, helper, &TracksListener::trackByIdInList);
     connect(client, &MediaPlayList::newTrackByNameInList, helper, &TracksListener::trackByNameInList);
+    connect(client, &MediaPlayList::newTrackByFileNameInList, helper, &TracksListener::trackByFileNameInList);
     connect(client, &MediaPlayList::newArtistInList, helper, &TracksListener::newArtistInList);
 }
 
@@ -199,8 +231,31 @@ ElisaApplication *MusicListenersManager::elisaApplication() const
     return d->mElisaApplication;
 }
 
+QAbstractItemModel *MusicListenersManager::allAlbumsModel() const
+{
+    return &d->mAllAlbumsModel;
+}
+
+QAbstractItemModel *MusicListenersManager::allArtistsModel() const
+{
+    return &d->mAllArtistsModel;
+}
+
+QAbstractItemModel *MusicListenersManager::allTracksModel() const
+{
+    return &d->mAllTracksModel;
+}
+
+bool MusicListenersManager::indexerBusy() const
+{
+    return d->mIndexerBusy;
+}
+
 void MusicListenersManager::databaseReady()
 {
+    d->mIndexerBusy = true;
+    Q_EMIT indexerBusyChanged();
+
     configChanged();
 }
 
@@ -232,11 +287,9 @@ void MusicListenersManager::resetImportedTracksCounter()
     }
 #endif
 
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
     for (const auto &itFileListener : d->mFileListener) {
         itFileListener->resetImportedTracksCounter();
     }
-#endif
 }
 
 void MusicListenersManager::setElisaApplication(ElisaApplication *elisaApplication)
@@ -266,13 +319,11 @@ void MusicListenersManager::playBackError(QUrl sourceInError, QMediaPlayer::Erro
 
 void MusicListenersManager::configChanged()
 {
-#if defined KF5Config_FOUND && KF5Config_FOUND
     auto currentConfiguration = Elisa::ElisaConfiguration::self();
 
     d->mConfigFileWatcher.addPath(currentConfiguration->config()->name());
 
     currentConfiguration->load();
-#endif
 
 #if defined KF5Baloo_FOUND && KF5Baloo_FOUND
     if (currentConfiguration->balooIndexer() && !d->mBalooListener) {
@@ -307,7 +358,6 @@ void MusicListenersManager::configChanged()
             &d->mUpnpListener, &UpnpListener::applicationAboutToQuit, Qt::DirectConnection);
 #endif
 
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
     if (currentConfiguration->elisaFilesIndexer())
     {
         const auto &allRootPaths = currentConfiguration->rootPath();
@@ -352,7 +402,6 @@ void MusicListenersManager::configChanged()
             }
         }
     }
-#endif
 }
 
 void MusicListenersManager::computeImportedTracksCount()
@@ -363,13 +412,18 @@ void MusicListenersManager::computeImportedTracksCount()
     } else {
         d->mImportedTracksCount = 0;
     }
+#else
+    d->mImportedTracksCount = 0;
 #endif
 
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
     for (const auto &itFileListener : d->mFileListener) {
         d->mImportedTracksCount += itFileListener->importedTracksCount();
     }
-#endif
+
+    if (d->mImportedTracksCount && d->mIndexerBusy) {
+        d->mIndexerBusy = false;
+        Q_EMIT indexerBusyChanged();
+    }
 
     if (d->mImportedTracksCount >= 4) {
         Q_EMIT closeNotification(QStringLiteral("notEnoughTracks"));
@@ -388,12 +442,14 @@ void MusicListenersManager::monitorStartingListeners()
     ++d->mActiveMusicListenersCount;
 }
 
-void MusicListenersManager::monitorEndingListeners()
+void MusicListenersManager::monitorEndingListeners(int tracksCount)
 {
     --d->mActiveMusicListenersCount;
 
+    d->mTotalImportedTracksCount += tracksCount;
+
     if (d->mActiveMusicListenersCount == 0) {
-        if (d->mImportedTracksCount < 4 && d->mElisaApplication) {
+        if (d->mTotalImportedTracksCount < 4 && d->mElisaApplication) {
             NotificationItem notEnoughTracks;
 
             notEnoughTracks.setNotificationId(QStringLiteral("notEnoughTracks"));
